package jdk.internal.org.objectweb.asm.commons;

import java.util.HashMap;
import java.lang.reflect.Constructor;
import jdk.internal.org.objectweb.asm.Type;
import java.util.Map;

public class Method
{
    private final String name;
    private final String desc;
    private static final Map<String, String> DESCRIPTORS;
    
    public Method(String name, String desc) {
        this.name = name;
        this.desc = desc;
    }
    
    public Method(String s, Type returnType, Type[] argumentTypes) {
        this(s, Type.getMethodDescriptor(returnType, argumentTypes));
    }
    
    public static Method getMethod(java.lang.reflect.Method method) {
        return new Method(method.getName(), Type.getMethodDescriptor(method));
    }
    
    public static Method getMethod(Constructor<?> constructor) {
        return new Method("<init>", Type.getConstructorDescriptor(constructor));
    }
    
    public static Method getMethod(String s) throws IllegalArgumentException {
        return getMethod(s, false);
    }
    
    public static Method getMethod(String s, boolean b) throws IllegalArgumentException {
        int index = s.indexOf(32);
        int n = s.indexOf(40, index) + 1;
        int index2 = s.indexOf(41, n);
        if (index == -1 || n == -1 || index2 == -1) {
            throw new IllegalArgumentException();
        }
        String substring = s.substring(0, index);
        String trim = s.substring(index + 1, n - 1).trim();
        StringBuilder sb = new StringBuilder();
        sb.append('(');
        int i;
        do {
            i = s.indexOf(44, n);
            String str;
            if (i == -1) {
                str = map(s.substring(n, index2).trim(), b);
            }
            else {
                str = map(s.substring(n, i).trim(), b);
                n = i + 1;
            }
            sb.append(str);
        } while (i != -1);
        sb.append(')');
        sb.append(map(substring, b));
        return new Method(trim, sb.toString());
    }
    
    private static String map(String anObject, boolean b) {
        if ("".equals(anObject)) {
            return anObject;
        }
        StringBuilder sb = new StringBuilder();
        int fromIndex = 0;
        while ((fromIndex = anObject.indexOf("[]", fromIndex) + 1) > 0) {
            sb.append('[');
        }
        String substring = anObject.substring(0, anObject.length() - sb.length() * 2);
        String str = (String)Method.DESCRIPTORS.get(substring);
        if (str != null) {
            sb.append(str);
        }
        else {
            sb.append('L');
            if (substring.indexOf(46) < 0) {
                if (!b) {
                    sb.append("java/lang/");
                }
                sb.append(substring);
            }
            else {
                sb.append(substring.replace('.', '/'));
            }
            sb.append(';');
        }
        return sb.toString();
    }
    
    public String getName() {
        return this.name;
    }
    
    public String getDescriptor() {
        return this.desc;
    }
    
    public Type getReturnType() {
        return Type.getReturnType(this.desc);
    }
    
    public Type[] getArgumentTypes() {
        return Type.getArgumentTypes(this.desc);
    }
    
    @Override
    public String toString() {
        return this.name + this.desc;
    }
    
    @Override
    public boolean equals(Object o) {
        if (!(o instanceof Method)) {
            return false;
        }
        Method method = (Method)o;
        return this.name.equals(method.name) && this.desc.equals(method.desc);
    }
    
    @Override
    public int hashCode() {
        return this.name.hashCode() ^ this.desc.hashCode();
    }
    
    static {
        (DESCRIPTORS = new HashMap()).put("void", "V");
        Method.DESCRIPTORS.put("byte", "B");
        Method.DESCRIPTORS.put("char", "C");
        Method.DESCRIPTORS.put("double", "D");
        Method.DESCRIPTORS.put("float", "F");
        Method.DESCRIPTORS.put("int", "I");
        Method.DESCRIPTORS.put("long", "J");
        Method.DESCRIPTORS.put("short", "S");
        Method.DESCRIPTORS.put("boolean", "Z");
    }
}
